/*
 * MixedMatrixMain.cpp	(CCFlib)
 * Copyright (C) 2004 Kento EMOTO
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Kento EMOTO:
 *   mailto:emoto@ipl.t.u-tokyo.ac.jp
 *   http://www.ipl.t.u-tokyo.ac.jp/~emoto/
 */
/** 
 *	@file MixedMatrixMain.cpp an example of using MixedMatrix (or calculating CCF).
 *	@author Kento EMOTO
 */
#include <iostream>
#include <iomanip>
#include <fstream>

using namespace std;

#include "CCF.hpp"
#include "bmp.hpp"

/**
 *	an example main function.
 *	do make and ./CCF -h for help
 */
int main(int argc, char *argv[])	// an expamle
{
	bool vflag = false;				// verbose output flag
	bool bflag = false;				// bitmap outout flag
	bool rflag = false;				// rank only flag
	bool tflag = false;				// set threshold flag
	MixedMatrix mm;					// mixed matrix
	
	while(argc > 1){				// parse parameters
		if(argv[1][0]=='-'){
			switch(argv[1][1]){
			case 'v': 
				vflag = true;
				break;
			case 'b':
				bflag = true;
				break;
			case 'r':
				rflag = true;
				break;
			case 'e':
				if(argc > 2){
					tflag = true;
					setEps(atof(argv[2]));
					argc--;
					argv++;
				}
				break;
			case 'h': 
				cout << "usage: CCF [options] < datafile\n";
				cout << "options:\n";
				cout << "\t-h\t: print this usage\n";
				cout << "\t-v\t: verbose\n";
				cout << "\t-b\t: output bitmap\n";
				cout << "\t-r\t: rank only\n";
				cout << "\t-e eps\t: set re-calculatioin threshold to eps\n\n";
				return 0;
			}
		}
		argc--;
		argv++;
	}
	mm.readFromStream(cin);				// read a mixed amtrix form standard in
	if(vflag){
		cout << "A =\n";
		mm.writeToStream(cout);			// print the matrix if vflag is true
	}
	mm.step1();								// STEP1
	if(rflag){
		mm.step2(false);					// STEP2 (and 3 (rank only))
		return 0;
	} else {
		mm.step2(true);						// STEP2 (and 3, 4 (CCF))
	}
	cout << "rank = " << mm.rank << "\n";
	if(vflag){
		cout << "CCF of A =\n";
		mm.writeToStreamCCF(cout);			// print CCF 
		cout << "S =\n";
		mm.writeMatrixToStream(cout, mm.matS);			// print S 
	} else {
		int i;								// print structure of CCF
		cout << "\n";
		cout << "Cs:";
		for(i = 0; i <= mm.blocknum+1; i++){
			cout << " " << mm.Cs[i];
		}
		cout << "\n";
		cout << "Rs:";
		for(i = 0; i <= mm.blocknum+1; i++){
			cout << " " << mm.Rs[i];
		}
		cout << "\n";
		cout << "\n";
	}
	if(bflag){										// output bitmap
		BMPManager bmp;
		int rowQ = mm.rowQ;							// rows of Q
		int rowT = mm.rowT;							// rows of T
		int row = rowQ + rowT;						// rows of Q + T
		int col = mm.col;							// columns of Q and T
		bmp.init(col, row);
		for(int i = 0; i < row; i++){
			for(int j = 0; j < col; j++){
				if(!mm.matCCFQ.get(i,j).isZero()){
					double d = mm.matCCFQ.get(i,j).doubleValue();
					if(d < 0){
						d = -d;
						int b = (int)(255 - d);
						if(b < 64) b = 64;
						bmp.setRGB(j, i, 0, 255, b);
					} else {
						int g = (int)(255 - d);
						if(g < 64) g = 64;
						bmp.setRGB(j, i, 0, g, 255);
					}
				} else if(!mm.matCCFT.get(i,j).isZero()){
					bmp.setRGB(j, i, 255, 0, 0);
				} else {
					bmp.setRGB(j, i, 255, 255, 255);
				}
			}
		}
		ofstream out;
		char fname[256];
		int j = sprintf(fname, "CCF");
		if(tflag){
			j += sprintf(fname + j, ".RT-%e", mm.getThreshold());
		}
		j += sprintf(fname + j, ".bmp");
		out.open(fname, ios::out | ios::binary);
		bmp.saveBMP(out);
		out.close();
	}
}
