/*
 * RationalNumber.cpp	(CCFlib)
 * Copyright (C) 2004 Kento EMOTO
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Kento EMOTO:
 *   mailto:emoto@ipl.t.u-tokyo.ac.jp
 *   http://www.ipl.t.u-tokyo.ac.jp/~emoto/
 */
/** 
 *	@file RationalNumber.cpp implementation of class RationalNumber.
 *	@author Kento EMOTO
 */
#include "RationalNumber.hpp"

/**
 *	canonicalize(make den and num coprime).
 */
void RationalNumber::canonicalize()
{
	int sign = 1;
	if(den < 0){
		den = -den;
		sign = -sign;
	}
	if(num < 0){
		num = -num;
		sign = -sign;
	}
	int u = gcd(num, den);
	num/=u;
	den/=u;
	num*=sign;
}
/**
 *	obtain gcd of x and y.
 *	@param x positive integer
 *	@param y positive integer
 */
int RationalNumber::gcd(int x, int y)
{
	while(y!=0){
		int r = x % y;
		x = y;
		y = r;
	}
	return x;
}
RationalNumber RationalNumber::operator+(const RationalNumber &rn)
{
	int s=1;
	int n, d, e;
	int u = gcd(den, rn.den);
	d = den/u;
	e = rn.den/u;
	n = num*e + rn.num*d;
	if(n==0){
		d = 1;
	} else {
		if(n < 0){
			s = -1;
			n = - n;
		}
		d*=rn.den;
		int u = gcd(d, n);
		d/=u;
		n/=u;
	}
	return RationalNumber(s * n, d);
}
RationalNumber RationalNumber::operator-(const RationalNumber &rn)
{
	int s=1;
	int n, d, e;
	int u = gcd(den, rn.den);
	d = den/u;
	e = rn.den/u;
	n = num*e - rn.num*d;
	if(n==0){
		d = 1;
	} else {
		if(n < 0){
			s = -1;
			n = - n;
		}
		d*=rn.den;
		int u = gcd(d, n);
		d/=u;
		n/=u;
	}
	return RationalNumber(s * n, d);
}
RationalNumber RationalNumber::operator*(const RationalNumber &rn)
{
	int u, v;
	if(rn.num < 0){
		u = gcd(den, - rn.num);
	} else {
		u = gcd(den, rn.num);
	}
	if(num < 0){
		v = gcd(- num, rn.den);
	} else {
		v = gcd(num, rn.den);
	}

	int n, d;
	d = (den/u)*(rn.den/v);
	n = (num/v)*(rn.num/u);
	return RationalNumber(n, d);
}
RationalNumber RationalNumber::operator/(const RationalNumber &rn)
{
	int u = gcd(den, rn.den);
	int v = gcd(num, rn.num);
	if(rn.num < 0){
		if(num < 0){
			v = gcd(-num, -rn.num);
		} else {
			v = gcd(num, -rn.num);
		}
	} else {
		if(num < 0){
			v = gcd(-num, rn.num);
		} else {
			v = gcd(num, rn.num);
		}
	}
	int n, d;
	d = (den/u)*(rn.num/v);
	n = (num/v)*(rn.den/u);
	if(d < 0){
		d = -d;
		n = -n;
	}
	return RationalNumber(n, d);
}
