/*
 * RationalNumber.hpp	(CCFlib)
 * Copyright (C) 2004 Kento EMOTO
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Kento EMOTO:
 *   mailto:emoto@ipl.t.u-tokyo.ac.jp
 *   http://www.ipl.t.u-tokyo.ac.jp/~emoto/
 */
/** 
 *	@file RationalNumber.hpp definition of class RationalNumber.
 *	@author Kento EMOTO
 */
#ifndef _RATIONAL_NUMBER_HPP_
#define _RATIONAL_NUMBER_HPP_

#include <iostream>
#include <iomanip>

using namespace std;

/**
 *	rational number.
 *	represented by (num, den), i.e. the value = num/den
 *	@author Kento EMOTO
 */
class RationalNumber {
public:
	int den;			/**< denominator */
	int num;			/**< numerator */
	
	/** constructor: initialized as zero */
	RationalNumber(){
		den = 1;
		num = 0;
	}
	/** constructor: initialized as integer value n */
	RationalNumber(int n){
		den = 1;
		num = n;
	}
	/** constructor: initialized as rational value n/d */
	RationalNumber(int n, int d){
		den = d;
		num = n;
	}
	/** operator taking RationalNumber rn(RHS) and returning this + rn */
	RationalNumber operator+(const RationalNumber &rn);
	/** operator taking RationalNumber rn(RHS) and returning this - rn */
	RationalNumber operator-(const RationalNumber &rn);
	/** operator taking RationalNumber rn(RHS) and returning this * rn */
	RationalNumber operator*(const RationalNumber &rn);
	/** operator taking RationalNumber rn(RHS) and returning this / rn */
	RationalNumber operator/(const RationalNumber &rn);
	/** operator returning - this */
	RationalNumber operator-(){
		return RationalNumber(-num, den);
	}
	void canonicalize();
	
	/** output to stream */
	friend ostream &operator<<(ostream &stream, RationalNumber rn){
		stream << rn.num << "/" << rn.den;
		return stream;
	}
	/** true if this value is zero */
	bool isZero() { return num==0; }
	/** return double value of this */
	double doubleValue() { return num/(double)den; }
	/** return int value of this denominator */
	int intDenValue() { return den; }
	/** return int value of this numerator */
	int intNumValue() { return num; }
protected:
	int gcd(int x, int y);
};

#endif
