/*
 * RealNumber.hpp	(CCFlib)
 * Copyright (C) 2004 Kento EMOTO
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Kento EMOTO:
 *   mailto:emoto@ipl.t.u-tokyo.ac.jp
 *   http://www.ipl.t.u-tokyo.ac.jp/~emoto/
 */
/** 
 *	@file RealNumber.hpp implementation and definition of class RealNumber.
 *	@author Kento EMOTO
 */
#ifndef _REALNUMBER_HPP_
#define _REALNUMBER_HPP_

/**
 * wrapper class of double.
 *	@author Kento EMOTO
 */
class RealNumber {
public:
	double d;			/**< value */
	/** constructor: initialized as zero */
	RealNumber(){
		d = 0;
	}
	/** constructor: initialized as integer value n */
	RealNumber(int n){
		d = n;
	}
	/** constructor: initialized as double value dd */
	RealNumber(double dd){
		d = dd;
	}
	/** operator taking RealNumber rn(RHS) and returning this + rn */
	RealNumber operator+(const RealNumber &rn){
		RealNumber r(d+rn.d);
		return r;
	}
	/** operator taking RealNumber rn(RHS) and returning this - rn */
	RealNumber operator-(const RealNumber &rn){
		RealNumber r(d-rn.d);
		return r;
	}
	/** operator taking RealNumber rn(RHS) and returning this * rn */
	RealNumber operator*(const RealNumber &rn){
		RealNumber r(d*rn.d);
		return r;
	}
	/** operator taking RealNumber rn(RHS) and returning this / rn */
	RealNumber operator/(const RealNumber &rn){
		RealNumber r(d/rn.d);
		return r;
	}
	/** operator returning - this */
	RealNumber operator-(){
		RealNumber r(-d);
		return r;
	}
	/** output to stream */
	friend ostream &operator<<(ostream &stream, RealNumber rn){
		stream << rn.d;
		return stream;
	}
	/** true if this value is zero */
	bool isZero() { return d==0; }
	/** return this value */
	double doubleValue() {return d; }
};
#endif
