/*
 * Variable.hpp	(CCFlib)
 * Copyright (C) 2004 Kento EMOTO
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Kento EMOTO:
 *   mailto:emoto@ipl.t.u-tokyo.ac.jp
 *   http://www.ipl.t.u-tokyo.ac.jp/~emoto/
 */
/** 
 *	@file Variable.hpp implementation and definition of class Variable.
 *	@author Kento EMOTO
 */
#ifndef _VARIABLE_HPP_
#define _VARIABLE_HPP_

/** independent parameter.
 *	@author Kento EMOTO
*/
class Variable {
public:
	double val;					/**< value of the parameter */
	/**
	 * an id of this parameter.
	 * if the id is zero, the parameter indicates zero. (is treated as zero)
	 */
	int id;
	/**
	 * constructor: initialized as zero
	 */
	Variable(){
		val = 0;
		id = 0;
	}
	/**
	 * constructor: id is initialized to i
	 * @param i id of this parameter
	 */
	Variable(int i){
		val = 0;
		id = i;
	}
	/**
	 * set id of this parameter
	 * @param i new id of this parameter
	 */
	void setId(int i){
		id = i;
	}
	/**
	 * true if this parameter is zero.
	 */
	bool isZero(){
		return id==0;
	}
	/**
	 *	output this parameter like 't' + id
	 */
	friend ostream &operator<<(ostream& stream, Variable v){
		if(v.id!=0){
			stream << 't' << v.id;
		} else {
			stream << 0;
		}
		return stream;
	}
};

#endif
