/*
 * VariableMatrix.hpp	(CCFlib)
 * Copyright (C) 2004 Kento EMOTO
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Kento EMOTO:
 *   mailto:emoto@ipl.t.u-tokyo.ac.jp
 *   http://www.ipl.t.u-tokyo.ac.jp/~emoto/
 */
/** 
 *	@file VariableMatrix.hpp implementation and definition of class VariableMatrix.
 *	@author Kento EMOTO
 */
#ifndef _VARIABLEMATRIX_HPP_
#define _VARIABLEMATRIX_HPP_
#include "Variable.hpp"

/** matrix of independent parameters
 * @see Variable
 *	@author Kento EMOTO
 */
class VariableMatrix {
public:
	Variable *ele;				/**< elements */
	int col;					/**< the number of columns */
	int row;					/**< the number of rows */
	
	/** 
	 *	constructor: initialized as 0x0 matrix
	 */
	VariableMatrix()
	{
		row = 0;
		col = 0;
		ele = NULL;
	}
	/** 
	 *	constructor: initialized as rxc matrix
	 *	@param r the number of rows
	 *	@param c the number of columns
	 */
	VariableMatrix(int r, int c)
	{
		row = r;
		col = c;
		ele = NULL;
		allocMemory();
	}
	/** 
	 *	initialize this matrix as rxc matrix
	 *	@param r the number of rows
	 *	@param c the number of columns
	 */
	void init(int r, int c){
		row = r;
		col = c;
		freeMemory();
		allocMemory();
	}
	/** destructor */
	~VariableMatrix()
	{
		freeMemory();
	}
	/** free memory */
	void freeMemory()
	{
		if(ele!=NULL) delete [] ele;
		ele = NULL;
	}
	/** allocate memory for elements */
	void allocMemory()
	{
		ele = new Variable[row*col];
	}
	/**
	 * get the elemtent of (i, j).
	 *	@param i row of the element (0 <= i < row)
	 *	@param j column of the element (0 <= j < col)
	 */
	Variable get(int i, int j){
		return ele[i*col + j];
	}
	/** 
	 * put an elemtent to (i, j).
	 *	@param i row of the element (0 <= i < row)
	 *	@param j column of the element (0 <= j < col)
	 *	@param dat Variable to be put to (i, j)
	 */
	void put(int i, int j, Variable dat){
		ele[i*col + j] = dat;
	}
};
#endif
